"""
base_data.py
其中包含BaseData类，用于存储数据
"""
import json
import os
from .setting import  CACHE_PATH, DATA_STORAGE_TYPE, DATA_DATA_TYPE, DATA_LANGUAGE, DATA_TASK_TYPE


class BaseData(object):
    def __init__(self, data_id: str):
        self.data_id = data_id
        self.config = self._load_config()
        self.data = self.load_data()



    def _load_config(self):
        """
        加载配置文件
        """

        config_path = os.path.join(CACHE_PATH, self.data_id, 'config.json')
        if not os.path.exists(config_path):
            print(f"配置文件config.json不存在，请检查路径：{config_path}是否正确")
            return False
        with open(config_path, 'r', encoding='utf-8') as f:
            print(f"配置文件config.json读取成功")
            return json.load(f)


    def get_name(self):
        name = self.config.get("name")
        print("name: ",end='')
        print(name)
        return name

    def get_url(self):
        url = self.config.get("url_or_path")
        print("url_or_path: ", end='')
        print(url)
        return url

    def get_tags(self):
        tags = self.config.get("tags")
        print("tags: ", end='')
        print("、".join(tags))
        return tags

    def get_category(self,number=False):
        cats = ['storage_type','data_type','language','task_type',]
        cats_name = [DATA_STORAGE_TYPE, DATA_DATA_TYPE, DATA_LANGUAGE,DATA_TASK_TYPE]
        if not number:
            cat = {c: cats_name[index].get(self.config.get(c)) for index, c in enumerate(cats)}
        else:
            cat = {c: self.config.get(c) for c in cats}
        print(cat)
        return cat

    def get_description(self):
        """
        数据使用说明,包括数据集说明和代码说明
        默认使用config.json中的description字段
        可自行重构方法
        """
        data_description = self.config.get('description')
        print("description: ",end='')
        print(data_description)
        return data_description

    def get_readme(self):
        md_path = os.path.join(CACHE_PATH, self.data_id, 'README.md')
        md = ''
        with open(md_path, 'r',encoding="utf-8") as f:
            md = f.read()
        print(f"README.md文件的路径为：{md_path}")

        return md



    def get_config(self,number=False):
        self.get_name()
        self.get_url()
        self.get_category(number=False)
        self.get_tags()
        self.get_description()



    def load_data(self):
        """
        !!! 必须重构方法
        加载数据
        可自行重构方法
        :return : dataset
        """

        pass

    def data_preview(self):
        """
        数据预览
        """
        pass

    def data_statistics(self):
        """
        描述性统计
        """
        pass

    def data_distribution(self):
        """
        数据分布
        """
        pass

    def data_preprocessing(self):
        """
        数据预处理
        """
        pass





import inspect
import json
import os.path

from .info import Information
from .setting import CACHE_PATH, INTERFACE_PATH,DATA_STORAGE_TYPE, DATA_DATA_TYPE, DATA_LANGUAGE, DATA_TASK_TYPE

import os
import sys
from .query import QueryDigit
import requests



import importlib.util


def find_py(path):
    # 获取指定路径下的所有文件
    files = os.listdir(path)

    # 检查是否存在 __init__.py 文件，如果不存在则创建
    init_file = "__init__.py"
    if init_file not in files:
        with open(os.path.join(path, init_file), "w") as f:
            pass

    # 查找名为 "code.py" 的文件
    if "code.py" in files:
        return "code"

    # 判断是否只有一个py文件
    py_files = [file for file in files if file.endswith(".py")]
    if len(py_files) == 1:
        return py_files[0].split(".")[0]


    return False


def check_config(config_path, qd):
    fp, fn = os.path.split(config_path)
    errors = []
    if fn!="config.json":
        errors.append("config文件名错误，必须为config.json")
        return errors


    cats = ['storage_type', 'data_type', 'language', 'task_type', ]
    cats_name = [DATA_STORAGE_TYPE, DATA_DATA_TYPE, DATA_LANGUAGE, DATA_TASK_TYPE]
    category = {c: cats_name[index] for index, c in enumerate(cats)}


    fields = ['name','url_or_path','storage_type','data_type','language','task_type','description','tags']

    fields_type = {
        'name':str,
        'url_or_path':str,
        'storage_type':int,
        'data_type':int,
        'language':int,
        'task_type':int,
        'description':str,
        'tags':list
    }


    with open(config_path, 'r', encoding='utf-8') as f:
        config = json.load(f)


    for field in fields:
        if field not in config:
            errors.append(f"{field}缺失")
            continue
        if type(config[field]) != fields_type[field]:
            errors.append(f"{field}字段值格式错误，应为{fields_type[field]}, 实际为{type(config[field])}")
            continue

        if field in fields[2:-2] :
            if config[field] not in category.get(field):
                errors.append(f"字段{field}的值为{config[field]},不在规定类型的数字代号范围中:{list(category.get(field).keys())}")

    return errors



class Data():

    def __init__(self):
        self.info = Information()
        self.qd = QueryDigit()
        self.data_id = ""


    def _get_data_id(self,data_id_or_name):
        # 检测是否已经缓存,
        qs = self.qd.get_resources(api_type="dataid", id=data_id_or_name)
        if not qs:
            print(f"data_id_or_name = {data_id_or_name}的资源不存在")
            return False
        data_id = qs[0].get('data_id')
        return data_id

    def load(self, data_id_or_name:str, imp_class="DigitData"):
        data_id = data_id_or_name
        path = os.path.join(CACHE_PATH, data_id_or_name)
        if not path:
            data_id = self._get_data_id(data_id_or_name=data_id_or_name)
            if not data_id:
                return False
            path = os.path.join(CACHE_PATH, data_id)
            if not os.path.exists(path):
                print(f"路径不存在：{path}")
                return False


        self.data_id = data_id

        code = find_py(path=path)
        if not code:
            print(f"未在该路径下找到代码文件")
            return False


        # 找到路径下的代码文件后
        # 动态加载
        # 添加路径，加载code.py文件，
        imp_class = imp_class
        sys.path.append(path)

        module_path = os.path.join(path,code)+".py"
        module_name = code
        spec = importlib.util.spec_from_file_location(module_name, module_path)
        imp_module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(imp_module)

        # ip_module = __import__('code')
        clss = inspect.getmembers(imp_module, inspect.isclass)
        print(clss)
        # 判断DigitData类是否存在

        clss_dict = {cls_name:cls_class for cls_name, cls_class in clss}
        if not imp_class in clss_dict:
            print("code.py文件中DigitData类不存在,请检查")
            return False

        if not inspect.getmro(clss_dict[imp_class])[1].__name__ in ['TableData', 'LabelData', 'DocData', 'ImageData',
                                                  'AudioData',
                                                  'VideoData', 'GraphData']:


            print("DigitData类不是规定类 TableData, LabelData, DocData, ImageData, AudioData, VideoData, GraphData 的子类")
            return False

        os.chdir(path)
        # 加载DIgitData类
        DD = getattr(imp_module, imp_class)  # 获取类
        return DD(data_id=data_id)  # 实例化类并返回



    def upload(self,md_instruction_path,config_path):
        """
        :param config_file:
        :param md_instruction_file:
        :param file_upload:
        :param kwargs:
        :return: data_id
        两种上传方式， 第一种文文件上传，第二种为字典格式上传
        """
        url  = f"{INTERFACE_PATH}/api/data/"
        headers = {'api-token':self.info.get_api_token()}


        errors = check_config(config_path=config_path, qd=self.qd)
        if errors:
            print(errors)
            return False


        files = {
            'cj': open(config_path, 'rb'),
            'md': open(md_instruction_path, 'rb')
        }
        response = requests.post(url=url, headers=headers, files=files)
        if response.status_code !=200:
            print(f"错误，错误代码为：{response.status_code}")
            return False

        content = response.json()

        if content['status']!=200:
            print(content.get('error'))
            return False

        print("上传成功")
        return content['data']








    def delete(self,data_id_or_name:str):

        data_id = self._get_data_id(data_id_or_name=data_id_or_name)
        if not data_id:
            return False

        url = INTERFACE_PATH + "/api/data/"
        headers = {'api-token':self.info.get_api_token()}
        response = requests.delete(url=url,headers=headers)
        if response.status_code != 204:
            print(f"删除时出现错误,错误代码{response.status_code}")
            return False
        content = response.json()

        if content.get("status") != 200:
            print(content.get('error'))
            return False

        print(content.get('msg'))
        return None # 成功删除

    def find_path(self):
        if not self.data_id:
            print("data_id为空")
            return 0
        path = os.path.join(CACHE_PATH, self.data_id)
        print(f"当前加载的代码所在路径为：{path}")
        return path





# 查询类，获取
# data, card，user, self, websetting, cache的查询

import os
import requests
from .setting import INTERFACE_PATH,DATA_STORAGE_TYPE, DATA_DATA_TYPE, DATA_LANGUAGE, DATA_TASK_TYPE
import time
from .info import Information
import pprint

class QueryDigit():

    def __init__(self, api_token=Information().get_api_token()):
        self.api_token = api_token
        self.queryset = ""
        self.api_path = INTERFACE_PATH
        self.api_types = ['data', 'card', 'user', 'account', 'websetting','dataid']

        self.api = {api_type: f"{INTERFACE_PATH}/api/{api_type}/" for api_type in self.api_types}

        self.errors = []
        self.brief = {
            "data": ['data_id', 'name', ],
            "card": ['card_id', 'name', ],
            "user": ['name', 'real_name'],
            "account": ['user_id', 'name', 'real_name', 'email'],
            "websetting":[],
            "dataid":['data_id']

        }

    def _get_all_resources(self, api_type):
        # api_type 取值为data, card, user,account, websetting
        status_code = 0
        max_request = 10  # 重复请求最大次数
        count = 0
        self.errors = []  # 每次获取新的数据都需要将之前的错误清空
        self.queryset = []
        response = ''
        url = self.api.get(api_type)
        # url = 'http://127.0.0.1:8000/api/user/'
        headers = {'api-token': self.api_token}

        while status_code != 200:  # 循环调用接口，避免因为网络故障导致的请求失败
            response = requests.get(url, headers=headers)
            status_code = response.status_code
            if status_code != 200:
                count += 1
                time.sleep(1)
            if count == max_request:
                self.errors.append(f"status code: {status_code}")
                return False

        content = response.json()

        if content.get('status') != 200:
            self.errors.append(content.get('error'))
            return False

        self.queryset = content.get('data')  # 将获取的数据存入queryset中
        return True  # 请求成功了

    def _get_single_resource(self, api_type: str, id):
        # 当api_type = dataid时， id表示的可能是data_id，也可能是name
        status_code = 0
        max_request = 10  # 重复请求最大次数
        count = 0
        self.errors = []  # 每次获取新的数据都需要将之前的错误清空
        self.queryset = []
        response = ''
        url = f"{self.api.get(api_type)}{id}/"

        headers = {'api-token': self.api_token}

        if api_type not in ['data', 'card', 'user','dataid']:
            self.errors.append(f"'{api_type}'的值只能为['data'、'card'、'user','dataid]四者之一")
            return False

        while status_code != 200:  # 循环调用接口，避免因为网络故障导致的请求失败
            response = requests.get(url, headers=headers)
            status_code = response.status_code

            if status_code != 200:
                count += 1
                time.sleep(1)
            if count == max_request:
                self.errors.append(f"status code: {status_code}")
                return False
        content = response.json()
        if content.get('status') != 200:
            self.errors.append(content.get('error'))
            return False
        self.queryset = [content.get('data')]  # 将获取的数据存入queryset中
        return True  # 请求成功了

    # 美化输出
    def _pretty_print(self, api_type: str):
        if api_type != "websetting":
            briefs = self.brief.get(api_type)
            print("\t".join(briefs))
            for resource in self.queryset:
                print("\t".join([resource[i] for i in briefs]))

    """
    已经写好获取两种类型的数据函数，下面是用户请求，选择是否输出details"""

    def get_resources(self, api_type: str, id=None, detail=False):
        if id:
            if not self._get_single_resource(api_type=api_type, id=id):
                print(self.errors[0])
                return self.queryset
        else:
            if not self._get_all_resources(api_type=api_type):
                print(self.errors[0])
                return self.queryset

        if not detail:
            self._pretty_print(api_type=api_type)
        else:
            print(self.queryset)

        return self.queryset



    def get_category(self):
        cats = ['storage_type', 'data_type', 'language', 'task_type', ]
        cats_name = [DATA_STORAGE_TYPE, DATA_DATA_TYPE, DATA_LANGUAGE, DATA_TASK_TYPE]
        category = {c:cats_name[index] for index, c in enumerate(cats)}
        pprint.pprint(category)
        return category


import json
import sys
import os

"""
加载info文件， 其中包括api_token和data的字典，如果已经缓存，则data字典中有对应data的data_id和data_name，
"""



class Information:
    def __init__(self):
        if getattr(sys, 'frozen', None): # 确保在不同环境下都能加载正确的文件路径， 无论是打包后的执行文件还是为打包的python脚本
            basedir = sys._MEIPASS
        else:
            basedir = os.path.dirname(__file__)
        self.file_path = os.path.join(basedir ,"info.json")
        self.api_token = ""
        self.data = {}
        self._load_data()

    def _load_data(self):
        with open(self.file_path) as json_file:
            data = json.load(json_file)
            self.data = data.get("data", {})
            self.api_token = data.get("api_token", "")

    def _save_data(self):
        try:
            with open(self.file_path, "w") as json_file:
                data = {
                    "api_token": self.api_token,
                    "data": self.data
                }
                json.dump(data, json_file, indent=4)
                print("api_token设置完成")
        except Exception as e:
            print(f"api-token设置出错，错误信息：{e}")

    def update_api_token(self, new_token):
        if new_token:
            self.api_token = new_token
            self._save_data()
        else:
            print("new_token 为空")

    def get_api_token(self):
        return self.api_token

    def add_data(self, data_id):
        # 增加已经下载的数据的缓存表，表示该数据已下载
        self.data[data_id] = data_id
        self._save_data()

    def delete_data(self, data_id):
        # 删除data_id对应的缓存判断表。查询不到，表示该数据未下载
        self.data.pop(data_id)
        self._save_data()

    def delete_all_data(self):
        self.data = []
        self._save_data()

    def query_data(self, data_id):
        if data_id in self.data:
            return self.data.get(data_id) # 如果存在则返回data_id
        return False # 如果不存在则返回False
